import UIComponent from "sap/ui/core/UIComponent";
import BCHLogger from "bchui5/util/BCHLogger";
import * as Debug from "bchui5/util/Debug";
import { getBchMetadata } from "bchui5/AppConfig";
import { getRootApp } from "bchui5/util/UI5Helpers";
import * as Connectivity from "bchui5/util/Connectivity";
import * as ConnectionCheck from "bchui5/util/http/ConnectionCheck";
import * as Device from "bchui5/util/Device";
import * as ComponentAccessor from "bchui5/util/ComponentAccessor";
import { requireAsync, requireSync, requireCallback } from "bchui5/util/Require";
import LocalStorage from "bchui5/util/storage/LocalStorage";
import JSONModel from "sap/ui/model/json/JSONModel";
import ResourceModel from "sap/ui/model/resource/ResourceModel";
import AuthUtil from "bchui5/util/auth/AuthUtil";
import * as Piwik from "bchui5/util/analytics/Piwik";
import "bchui5/BCHRouter";
export { requireAsync, requireSync };
const logger = new BCHLogger("BCHComponent");
const ICON_DEFAULTS = {
    "menu": "sap-icon://menu2",
    "feedback": "sap-icon://travel-request",
    "logout": "sap-icon://employee-rejections",
    "favourites": "sap-icon://favorite",
    "calendar": "sap-icon://calendar",
    "private": "sap-icon://private",
    "back": "sap-icon://nav-back"
};
/**
 * @name bchui5.Component
 */
export default class BCHComponent extends UIComponent {
    constructor() {
        super(...arguments);
        this.requireSync = requireSync;
        this.requireAsync = requireAsync;
    }
    getAppId() {
        return this.getAppConfig().appId;
    }
    getAuthUtil() {
        return this._authUtil;
    }
    getAuthModel() {
        return this.getModel("auth");
    }
    getDeviceModel() {
        return this.getModel("device");
    }
    async whenCoreInitialized() {
        // Resolves when UI5 Core is initialized.
        return await this._whenCoreInitialized;
    }
    async whenInitialized() {
        return await this._whenInitialized;
    }
    async whenRendered() {
        // Resolves after the App control is rendered.
        return await this._whenAppRendered;
    }
    triggerAppRendered() {
        // Triggered by AbstractAppController after app is rendered.
        if (this._appRenderedResolve) {
            this._appRenderedResolve();
            this._appRenderedResolve = undefined;
        }
    }
    whenDeviceReady() {
        return Device.whenReady();
    }
    setActive() {
        ComponentAccessor.setActive(this);
    }
    getRootApp() {
        return this.app;
    }
    init(...args) {
        logger.start("init");
        ComponentAccessor.registerComponent(this); // Need to do this first in case an app uses ComponentAccessor at module level.
        const appConfig = this.getAppConfig();
        this.appConfig = appConfig; // Kept for backwards compatibility. Although hopefully all apps use getAppConfig()
        this.initInitializationPromises();
        if (Debug.isEnabled() || (appConfig.target && appConfig.target.includes("dev"))) {
            // Register global when in dbg mode so developers are aware of JS errors
            // Also register in 'dev' target since some errors that first happen in 'dev' mode are hard to diagnose.
            requireCallback("bchui5/util/error/GlobalErrorHandler", (GlobalErrorHandler) => {
                GlobalErrorHandler.registerGlobal();
            });
        }
        Debug.add({ component: this });
        initConfigModel(this);
        initI18nModel(this);
        initIconModel(this);
        if (appConfig.service) {
            ConnectionCheck.init(appConfig);
            const connectivityModel = Connectivity.init(this, ConnectionCheck);
            this.setModel(connectivityModel, "connectivity");
        }
        this.setModel(Device.init(), "device");
        Piwik.init(appConfig);
        this._authUtil = AuthUtil.init(this); // Sets up subscriptions.
        conditionallyInitHybrid(this);
        initNetworkObserver(this);
        // Make sure to declare this._whenInitialized before calling UIComponent.prototype.init()
        this._whenInitialized = this.whenCoreInitialized()
            .then(() => startAfterCoreInitializedFlow(this))
            .catch((ignore) => logger.caught(ignore));
        super.init(...args);
        this.app = getRootApp(this); // Needs to be after UIComponent.prototype.init
        this.getRouter().initialize();
        logger.complete("init");
    }
    // exit(...args: any[]) {
    //   return UIComponent.prototype.exit.apply(this, args)
    // }
    initInitializationPromises() {
        // Core initialization needs to complete before any dialogs can be rendered.
        const core = sap.ui.getCore();
        if (core.isInitialized()) {
            this._whenCoreInitialized = Promise.resolve();
        }
        else {
            this._whenCoreInitialized = new Promise(resolve => core.attachInit(resolve));
        }
        // Set a rendering promise on the component so that the modules can easily know when the app has rendered.
        // This will determine when it is safe to display things like dialogs
        // since InstanceManager calls closeAllDialogs() after the router initializes.
        this._whenAppRendered = new Promise(resolve => {
            this._appRenderedResolve = resolve;
        });
    }
}
BCHComponent.metadata = getBchMetadata();
function initConfigModel(component) {
    const appConfig = component.getAppConfig();
    component.setModel(new JSONModel(appConfig), "config");
}
async function startAfterCoreInitializedFlow(component) {
    try {
        const appConfig = component.getAppConfig();
        const storage = new LocalStorage({
            parse: true
        });
        if (appConfig.privacy && appConfig.privacy.auto) {
            const version = appConfig.privacy.version || 1;
            const privacyShown = storage.getItem("privacyShown"); // bool or int
            if (!privacyShown) {
                const PrivacyInfoDialog = await import("bchui5/util/controls/PrivacyInfoDialog");
                await PrivacyInfoDialog.openAndAwaitClose();
                storage.setItem("privacyShown", appConfig.privacy.version || true);
            }
            else if (privacyShown < version) {
                // If privacy has been shown, but the version was updated, show the warning dialog
                const PrivacyUpdatedDialog = await import("bchui5/util/controls/PrivacyUpdatedDialog");
                await PrivacyUpdatedDialog.openAndAwaitClose();
                storage.setItem("privacyShown", version);
            }
        }
        if (appConfig.tutorial && appConfig.tutorial.auto) {
            if (!storage.getItem("tutorialShown")) {
                const TutorialDialog = await import(`${"bchui5/util/controls/TutorialDialog"}`);
                await TutorialDialog.openAndAwaitClose();
                storage.setItem("tutorialShown", true);
            }
        }
        if (appConfig.login) {
            await component.getAuthUtil().doInitialLogin();
        }
    }
    catch (error) {
        console.error(error);
        alert(`Error initializing app. Please notify support.\n${error.message}`);
    }
}
function conditionallyInitHybrid(component) {
    if (Device.isFileApp()) {
        // lazy require in case we never reach here
        requireSync("bchui5/util/cordova/Cordova").init(component);
        if (window.screen.height < 500) {
            jQuery("html").addClass("bchShort");
        }
        if (window.FastClick) {
            window.FastClick.attach(document.body);
        }
        document.addEventListener("deviceready", onHybridDeviceReady);
    }
}
function onHybridDeviceReady() {
    const nav = navigator;
    if (nav.splashscreen) {
        nav.splashscreen.hide();
    }
    if (Debug.isEnabled()) {
        const versions = {
            cookies: 1.35,
            EmailComposer: 1.30
        };
        const lessThanOrUndefined = (actual, expected) => {
            return !actual || actual < expected;
        };
        if (window.cookies && lessThanOrUndefined(window.cookies.version, versions.cookies)) {
            return Debug.alert(`Update the cookies plugin to at least ${versions.cookies}.`);
        }
        if (!window.EmailComposer) {
            return Debug.alert(`Install the EmailComposer plugin.`);
        }
        else if (lessThanOrUndefined(window.EmailComposer.version, versions.EmailComposer)) {
            return Debug.alert(`Update the EmailComposer plugin to at least ${versions.EmailComposer}.`);
        }
        // Others (cordovaHTTP, BCHNetwork, Keychain) are checked in their own utils
    }
}
function initNetworkObserver(component) {
    if (component.getAppConfig().service && Device.isFileOrLocalhost()) {
        Device.afterDeviceReady(async () => {
            const NetworkObserver = await import("bchui5/util/cordova/NetworkObserver");
            NetworkObserver.init(component);
        });
    }
}
/**
 * Helper for the icon model.
 */
function initIconModel(component) {
    // appConfig.icons can be an object, false, or undefined.
    // If an object, just use that.
    // If false, don't try loading a separate json. If undefined, try loading the json.
    const appConfig = component.getAppConfig();
    const appIcons = (appConfig.icons === false) ? {}
        : appConfig.icons ? appConfig.icons
            : jQuery.sap.syncGetJSON(jQuery.sap.getModulePath(appConfig.appModule) + "/config/icon.json").data;
    const icons = { ...ICON_DEFAULTS, ...appIcons };
    const model = new JSONModel(icons);
    sap.ui.getCore().setModel(model, "icon"); // The core icon model is used by the dialogs. TODO remove core model usage.
    component.setModel(model, "icon"); // The component icon model is used used by app specific (and also common) menu items.
}
/**
 * Helper for the i18n (internationalization) model.
 */
function initI18nModel(component) {
    const appConfig = component.getAppConfig();
    if (appConfig.i18n === false) {
        return;
    }
    // Always use absolute paths relative to our own component
    // (relative paths will fail if running in the Fiori Launchpad)
    const i18nModel = new ResourceModel({
        // This is relative to the app's Component.js
        bundleUrl: `${appConfig.appModulePath}/i18n/i18n.properties`
    });
    component.setModel(i18nModel, "i18n");
}
