
import AbstractController from "bchui5/mvc/controller/AbstractController";
import BCHLogger from "bchui5/util/BCHLogger";
import Debug from "bchui5/util/Debug";
import { AuthEventIds } from "bchui5/util/auth/AuthInterfaces";
import { ModelNames } from "exceptiontime/util/Enums";

import CalendarEventType from "sap/me/CalendarEventType";
import CalendarDate from "sap/me/CalendarDate";

const logger = new BCHLogger("Calendar.controller");

const CalendarSelectionMode = sap.me.CalendarSelectionMode; // this can't be imported

const Legend = {
  Existing: CalendarEventType.Type04
};

/**
 * @name exceptiontime.controller.Calendar
 * @controller
 */
export default class CalendarController extends AbstractController {

  calendar = this.byId("calendar");
  selectButton = this.byId("selectBtn");
  clearButton = this.byId("clearBtn");
  modeSelect = this.byId("modeSelect");

  /**
   * Controller init
   */
  onInit() {

    this.calendar.setSelectionMode(CalendarSelectionMode.SINGLE);

    const component = this.getComponent();

    component.getAuthUtil()
      .observeEvents()
      .filter(event => event.name === AuthEventIds.LoggedOut)
      .subscribe(this.handleLoggedout.bind(this));

    this.getEventBus().subscribe("odata", "entries", this.highlightExistingEntryDates, this);

    Debug.add({
      calendar: this.calendar
    });
  }

  /**
   * Called before each time the view is shown,
   * including from both forward and backward navigation.
   * And also called for a split-app 'refresh'
   *
   * @param {$.Event} evt
   *   {object} [data] - contains data from forward navigation
   *   {object} [backData] - may contain data passed back
   *   {string} direction - "to", "back", "initial" or "refresh"
   *   ...more...
   */
  onBeforeShow() {
    if (this.isSingleMode()) {
      if (this.calendar.getSelectedDates().length === 0) {
        // set the current date as the 'previously' selected. mainly just to simplify the legend
        this.calendar.toggleDatesSelection([new CalendarDate()], true);
      }
    }
    this.enableOrDisableSelect();
  }

  /**
   * Called before the first time the view is shown.
   * See onBeforeShow for params
   */
  onBeforeFirstShow() {
    this.highlightExistingEntryDates();
  }

  /**
   * Handle button press event (Back/Cancel button)
   */
  onCancelPress() {
    // TODO in multi mode, revert to the dates that were selected when they opened the calendar.
    // Timeout for a bit to give the user a second to lift their finger before the nav-ing back (maybe just a chrome emulation issue)
    setTimeout(this.getRouter().navBack, 10);
  }

  /**
   * Handle button press event (Select button)
   */
  onSelectPress() {
    const selectedDates = (this.calendar
      .getSelectedDates()
      .map(date => new Date(date))
      .sort((a, b) => a - b)
    );
    this.getRouter().navBack({
      data: {
        selectedDates
      }
    });
  }

  /**
   * Handle button press event (Clear button)
   */
  onClearPress() {
    this.calendar.unselectAllDates();
    this.enableOrDisableSelect();
  }

  /**
  * Handle a date change on the cal.
  * Note used since we use the tap event.
  **/
  onDateChange() {}

  /**
   * Handle a single date tap on the calendar
   */
  onDateTap(evt) {
    logger.start("onDateTap");
    if (this.isSingleMode()) {
      // When the user selects the currently selected date, it actually un-selects it.
      // That's not the behaviour we want, so we need to re-select it on the calendar.
      // Then 'onSelectPress' can get it from the calendar.
      if (!evt.getParameter("didSelect")) {
        this.calendar.toggleDatesSelection([evt.getParameter("date")], true);
      }
      this.onSelectPress();
    }
    else {
      this.enableOrDisableSelect();
    }
  }

  /**
   * Handle a swipe event on the calendar
   */
  onChangeRange() {
    logger.start("onChangeRange");
    this.enableOrDisableSelect();
  }

  /**
   * Handle a select/option change for the calendar mode
   * @binding
   **/
  onDateModeChange() {
    if (this.isSingleMode(this.modeSelect.getSelectedKey())) {
      this.setSingleMode();
    }
    else {
      this.setMultiMode();
    }
  }

  /**
   * Check whether the passed in val or the calendar is Single mode.
   * @return {boolean}
   * @private
   */
  isSingleMode(val) {
    val || (val = this.calendar.getSelectionMode());
    return (val === CalendarSelectionMode.SINGLE);
  }

  /**
   * Convert the view into single mode,
   * including the calendar itself and anything else such as buttons.
   * @private
   */
  setSingleMode() {
    this.calendar.setSelectionMode(CalendarSelectionMode.SINGLE);
    this.calendar.unselectAllDates();
    this.selectButton.setVisible(false);
    this.clearButton.setVisible(false);
  }

  /**
   * Convert the view into multi mode,
   * including the calendar itself and anything else such as buttons.
   * @private
   */
  setMultiMode() {
    this.calendar.setSelectionMode(CalendarSelectionMode.MULTIPLE);
    this.calendar.unselectAllDates();
    this.selectButton.setVisible(true);
    this.clearButton.setVisible(true);
    this.enableOrDisableSelect();
  }

  /**
   * Disabled/Enable the Select and Clear buttons.
   * Used after a tap or swipe, on load, and and mode change
   * @private
   */
  enableOrDisableSelect() {
    const hasSelected = this.calendar.getSelectedDates().length > 0;
    this.selectButton.setEnabled(hasSelected);
    this.clearButton.setEnabled(hasSelected);
  }

  /**
   * Highlight the existing entries using Calendar date type 04
   * @private
   */
  highlightExistingEntryDates() {
    if (this.entryDates) {
      this.clearExistingEntries();
    }
    const entries = this.getComponent().getModel(ModelNames.CurrentEntries).getEntries();
    const entryDates = entries.map(entry => new Date(entry.Date)); // clone the date to avoid mutation
    // slice to clone array, so entryDates Date objects are not changed to Strings
    this.calendar.toggleDatesType(entryDates.slice(0), Legend.Existing, true);
    this.entryDates = entryDates; // save them so we can revert later
  }

  /**
   * Clear all the existing entries from the Calendar
   * @private
   */
  clearExistingEntries() {
    this.calendar.toggleDatesType(this.entryDates, Legend.Existing, false);
  }

  /**
   * Handle logout bus event to clear reset the calendar.
   * @private
   */
  handleLoggedout() {
    this.clearExistingEntries();
  }

}
