
import AbstractController from "bchui5/mvc/controller/AbstractController";
import BCHLogger from "bchui5/util/BCHLogger";
import FlashDialog from "bchui5/util/controls/FlashDialog";
import * as Device from "bchui5/util/Device";
import * as EventBus from "bchui5/util/EventBus";
import ChargeCodes from "com/bchydro/findmychargecode/mvc/model/ChargeCodes";
import * as FavsService from "com/bchydro/findmychargecode/mvc/service/FavsService";
import * as HttpService from "com/bchydro/findmychargecode/mvc/service/HttpService";

/* global CBuffer */
jQuery.sap.require("com.bchydro.findmychargecode.libraries.cbuffer");

const NAME = "Details.controller";
const logger = new BCHLogger(NAME);

/**
 * @name com.bchydro.findmychargecode.mvc.controller.Details
 */
export default class DetailsController extends AbstractController {

  constructor() {
    super();
    // stores the charge code
    this.chargeCode = null;
  }

  // flag whether the same code was clicked again.
  // Set in before show and checked in after show
  _sameAsLastCode = false;

  // search and request caching
  cache = {
    cbuffer: null,
    children: {},
    parent: {}
  };

  onInit() {
    logger.start("onInit");
    this.details = this.byId("details");
    this.details.setBusyIndicatorDelay(0);
    this.parentList = this.byId("parentList");
    this.childrenList = this.byId("childrenList");
    this.favBtn = this.byId("favButton");
    this.page = this.byId("page");
    this.pageContent = this.byId("pageContent");
    this.i18n = this.getComponent().getModel('i18n');
    this.model = new ChargeCodes({});
    this.getView().setModel(this.model);

    this.getEventBus().subscribe("app", "logout", () => {
      this.pageContent.setVisible(false); // it's hidden initially too
    });

    if (Device.isHybridOrLocalhost()) {
      // need a circular buffer on hybrid since the app stays open all the time.
      // get it on localhost too just for testing
      this.cache.cbuffer = new CBuffer(15);
    }
  }

  onBeforeFirstShow() {
    logger.start("onBeforeFirstShow");
  }

  /**
     * @param evt:
     *	Event hash evt.data: (only when called from nav-to) contains
      *	the document evt.fromId: optional id of the view which
      *	initiated the refresh
      */
  onBeforeShow(evt) {
    logger.start("onBeforeFirstShow", evt.direction);
    if (evt.direction !== "initial") {
      this._showContent();
      this.refreshView(evt);
    }
  }

  onAfterShow(evt) {
    logger.start("onAfterShow", evt.direction);
    // initial is when the page gets loaded in a split app
    if (!this._sameAsLastCode && evt.direction !== "initial") {
      this._getParent();
      this._getChildren();
    }
  }

  /**
     * Refresh the view. Typically on navigation, but also called for split apps.
     * @param @see onBeforeShow()
     */
  refreshView(evt) {
    logger.start("refreshView");
    const data = evt.data;
    const chargeCode = data.active;
    const oldChargeCode = this.chargeCode;
    if (oldChargeCode && oldChargeCode.ChargeCodeKey === chargeCode.ChargeCodeKey) {
      this._sameAsLastCode = true;
      this._flicker();
      return;
    }
    this.chargeCode = chargeCode;
    this._sameAsLastCode = false;
    // prepare the noDataText with the word 'Searching' before calling setData() which will clear the lists
    this._prepareParentText();
    this._prepareChildrenText();
    chargeCode.Parent = [];
    chargeCode.Children = [];
    const model = this.model;
    model.setData(chargeCode);
    model.refresh();
    // this.getView().setModel(new JSONModel(this.chargeCode));
    this._toggleFavIcon(this.chargeCode.isFav);
    cycleCache(this.cache, chargeCode);
    // The actual load time is pretty much instantaneous, but we keep the busy
    // dialog because its hard to see that the information has changed (Just to make it clear to the user)
    this._flicker();
  }

  /**
     * Back button handler
     */
  onNavBackPress() {
    logger.start("onNavBackPress");
    this.getRouter().navBack();
  }

  /**
     * List item handler (Parent or Child).
     *
     * @param {sap.ui.base.Event} evt - The UI5 event
     * 	@param {sap.m.Control} source - the control that was pressed.
     */
  async onListItemPress(evt) {
    logger.start("onListItemPress");
    const clickedItem = evt.getSource();
    const context = clickedItem.getBindingContext();
    const itemData = context.getObject();
    this.refreshView({
      data: {
        active: itemData
      }
    });
    await Promise.all([
      this._getParent(),
      this._getChildren()
    ]);
  }

  /**
     * Called when user clicks the fav button
     */
  onFavIconPress() {
    logger.start("onFavIconPress");
    const chargeCode = this.chargeCode;
    const isFav = FavsService.toggle(chargeCode);
    const eventId = isFav ? "add" : "remove";

    EventBus.tryPublish(this.getEventBus(), "favs", eventId, chargeCode);
    this._toggleFavIcon(isFav);
  }

  /**
   * Fav button
   */
  _toggleFavIcon(on) {
    logger.start("toggleFavIcon", on);
    this.favBtn.setIcon(`sap-icon://${on ? "favorite" : "unfavorite"}`);
  }

  /**
   * show the content. After forward navigation.
   * @private
   */
  _showContent() {
    logger.start("showContent");
    this.pageContent.setVisible(true); // it's hidden initially
  }

  /**
     * Prepares the parent no data text before the view is shown.
     * To be used from onBeforeShow so that the old value is not shown.
     */
  _prepareParentText() {
    logger.start("prepareParentText");
    // searching will happen in onAfterShow, so set 'Searching' now for the transition
    if (this.chargeCode.hasParent) {
      this.parentList.setNoDataText(this.i18n.getProperty("searching"));
    }
  }

  /**
     * Prepares the children no data text before the view is shown.
     */
  _prepareChildrenText() {
    logger.start("prepareChildrenText");
    if (ChargeCodes.isParentCandidate(this.chargeCode)) {
      this.childrenList.setNoDataText(this.i18n.getProperty("searching"));
    }
  }

  _resetParentText() {
    this.parentList.setNoDataText(this.i18n.getProperty("parent_no_data"));
  }

  _resetChildrenText() {
    this.childrenList.setNoDataText(this.i18n.getProperty("child_no_data"));
  }

  _flicker() {
    const details = this.details;
    details.setBusy(true);
    setTimeout(() => {
      details.setBusy(false);
    }, 50);
  }
  
  /**
     * Make the service call to get the parent. Do caching here too.
     */
  async _getParent() {
    logger.start("getParent");
    const thisList = this.parentList;
    const chargeCode = this.chargeCode;
    const thisCache = this.cache.parent;
    const cacheKey = getCacheKey(chargeCode);
  
    if (!chargeCode.hasParent) {
      this._resetParentText();
      return;
    }
  
    console.log("Checking parent cache for key:", cacheKey);
    const cachedResponse = thisCache[cacheKey];
    if (cachedResponse) {
      // debugger;
      this._resetParentText();
      console.log("Using cached parent for:", cachedResponse);
      this.model.setParentFromSearch(cachedResponse);
      return;
    }
  
    thisList.setBusyIndicatorDelay(0);
    thisList.setNoDataText(this.i18n.getProperty("searching"));
  
    const searchCodeKey = chargeCode.ChargeCodeKey;
    const searchCodeType = chargeCode.ChargeCodeType;
    
    try {
      const response = await HttpService.get({
        busy: thisList,
        path: "ChargeCodeCollection(ChargeCodeType='" + searchCodeType + "',ChargeCodeKey='" + searchCodeKey + "')/Parent",
        // this shouldShowError option is specific to this app's HttpService
        shouldShowError: shouldShowErrorsCheck
      });
      thisCache[cacheKey] = response;
      this.model.setParentFromSearch(response);
    }
    catch (error) {
      // HttpService will have shown the errors if needed.
      FlashDialog.unhandled(error);
      this.model.setParent([]);
    }
    finally {
      // always revert it from the 'Searching' text
      this._resetParentText();
    }
  }
  
  /**
   * Make the service call to get the children. Do caching here too.
   */
  async _getChildren() {
    logger.start("getChildren");
  
    const thisList = this.childrenList;
    const chargeCode = this.chargeCode;
    const thisCache = this.cache.children;
    const cacheKey = getCacheKey(chargeCode);
  
    if (ChargeCodes.isNotParentCandidate(chargeCode)) {
      this._resetChildrenText();
      return;
    }
  
    console.log("Check child cache for cacheKey:", cacheKey);
    const cachedResponse = thisCache[cacheKey];
    if (cachedResponse) {
      // debugger;
      this._resetChildrenText();
      console.log("Cached children found:", cachedResponse);
      this.model.setChildrenFromSearch(cachedResponse);
      // cachedModel.refreshFavsProperty();
      // need to copy the model or else the list may not update
      // thisList.setModel(cachedModel.copy());
      return;
    }
  
    thisList.setBusyIndicatorDelay(0);
    thisList.setNoDataText(this.i18n.getProperty("searching"));
  
    const searchCodeKey = chargeCode.ChargeCodeKey;
    const searchCodeType = chargeCode.ChargeCodeType;
    
    try {
      const response = await HttpService.get({
        path: "ChargeCodeCollection(ChargeCodeType='" + searchCodeType + "',ChargeCodeKey='" + searchCodeKey + "')/Children",
        busy: thisList,
        // this shouldShowError option is specific to this app's HttpService
        shouldShowError: shouldShowErrorsCheck
      });
      thisCache[cacheKey] = response;
      this.model.setChildrenFromSearch(response);
    }
    catch (error) {
      FlashDialog.unhandled(error);
      this.model.setChildren([]);
    }
    finally {
      this._resetChildrenText();
    }
  }

}


function cycleCache(cache, chargeCode) {
  logger.start("cycleCache", chargeCode);
  const cbuffer = cache.cbuffer;
  if (!cbuffer) {
    return; // no circular buffer used on desktop
  }
  const cacheKey = getCacheKey(chargeCode);

  // TODO implement a touch method in the buffer that moves the key to the back
  // otherwise, a recently used key might get bumped
  const includesKey = cbuffer.includes(cacheKey);
  if (includesKey) {
    return;
  }
  // push the new key to the buffer and see if one got bumped
  const bumped = cbuffer.push(cacheKey);
  if (bumped) {
    // if a key has been bumped from the buffer,
    // also delete its saved search results
    delete cache.parent[bumped];
    delete cache.children[bumped];
  }
}


function getCacheKey(chargeCode) {
  return chargeCode.ChargeCodeKey + "-" + chargeCode.ChargeCodeType;
}

function shouldShowErrorsCheck(error) {
  if (error.status === 404) {
    logger.caught(error);
    return false; // suppress default handler/flash
  }
  else if (error.status === 500 && error.responseText.includes("This row is not contained in the table")) {
    logger.caught(error);
    return false; // suppress default handler/flash
  }
  else {
    // trigger default handler
    return true;
  }
}
