"use strict";
import '../interfaces';
import JSONModel from "sap/ui/model/json/JSONModel";
import SAPDevice from "sap/ui/Device";
import Connectivity, { // type
isOnline, isOffline, whenOnline, whenOffline, whenStateChange, onStateChange, onSingleStateChange, } from "bchui5/util/Connectivity";
const BindingMode = sap.ui.model.BindingMode;
export { isOnline, isOffline, whenOnline, whenOffline };
//Keep the BCHUI5 dependencies here minimal.
/**
 * Device or app util methods.
 * All functions should be completely stateless.
 * Warning: Model number functions depend on the cordova device plugin (org.apache.cordova.device)
 */
let deviceReadyPromise;
let isReady = false;
export function whenDeviceReady() {
    return deviceReadyPromise;
}
/**
 * Checks whether an app is running on localhost
 * @returns {Boolean}
  */
export function isLocalhost() {
    return window.location.hostname === "localhost";
}
/**
 * Checks whether an app page is served from file.
 * Different than isHybrid because a launchpad app may have window.cordova
 * but not served from file.
 * This one is also safe to call before cordova is ready, where window.cordova may not be.
 * @returns {Boolean}
 */
export function isFileApp() {
    return window.location.protocol === "file:";
}
export function isFileHybrid() {
    return window.location.protocol === "file:";
}
/**
 * Checks whether an app is hybrid, if it meets either condition
 * of having window.cordova or being served from file.
 * Returns true for an https served html like launchpad inside a cordova container.
 * Be careful with the expected result of launchpad based apps (true).
 * @returns {Boolean}
 */
export function isHybrid() {
    return window.location.protocol === "file:" || !!window.cordova;
}
/**
 * A hybrid launchpad app (i.e. inside Fiori launchpad)
 * will be served over http/https, rather than file.
 * @returns {Boolean}
 **/
export function isHybridLaunchpadApp() {
    if (isFileApp()) {
        return false;
    }
    //else http/https
    try {
        if (window.cordova || window.parent.cordova)
            return true;
    }
    catch (error) {
        //catch and ignore SecurityError when running in an iframe on portal
    }
    //if (window.parent.cordova) return true; //works on iOS 7 & 8
    //since we can't check for window.cordova or window.parent.cordova on iOS9
    //our solution gets a bit hacky.
    if (SAPDevice.system.desktop)
        return false;
    //if we're here then we're on a phone or tablet.
    //But it could be mobile browser rather than launchpad.
    //It's a bit hacky but check for SMP urls.
    if (window.location.hostname.match(/mdm.bchydro.com/))
        return true;
    if (window.location.hostname.match(/smp.bchydro.bc.ca/))
        return true;
}
/**
 * Checks whether an app is hybrid or running on localhost
 * @returns {Boolean}
 */
export function isHybridOrLocalhost() {
    return isHybrid() || isLocalhost();
}
/**
 * Checks whether an app is file: based or running on localhost
 * Different from isHybridOrLocalhost as it will be false
 * for remote served launchpad apps
 * @returns {Boolean}
 */
export function isFileOrLocalhost() {
    return isFileApp() || isLocalhost();
}
/**
 * Checks whether an app is desktop browser based.
 * false for localhost and hosted Lauchpad apps.
 * @returns {Boolean}
 */
export function isDesktopBrowser() {
    return !isHybridOrLocalhost();
}
export function getRuntime() {
    // These runtimes match those used in app.json
    // Which is why some may not be semantically correct (i.e. hybrid vs hybrid launchpad)
    if (isFileApp()) {
        return "hyrbid";
    }
    else if (isLocalhost()) {
        return "localhost";
    }
    else {
        // Also applies to fiori launchpad on browser and on SAP Apps
        return "server";
    }
}
function getEnvType() {
    return getRuntime();
}
const Device = {
    ...SAPDevice,
    init() {
        const model = initDeviceModel();
        initDeviceReady();
        return model;
    },
    isHybrid,
    isLocalhost,
    isFileOrLocalhost,
    isFileApp,
    isFileHybrid,
    isHybridLaunchpadApp,
    isHybridOrLocalhost,
    isDesktopBrowser,
    getRuntime,
    getEnvType,
    getDescriptiveName() {
        if (isHybrid()) {
            return Device.os.name + " " + Device.os.versionStr;
        }
        else {
            return Device.browser.name.toUpperCase() + " " + Device.browser.versionStr;
        }
    },
    // getBrowserName() {
    //   const shortName = Device.browser.name; //i.e. 'cr' or 'ff' or 'ie'
    //   // const map = Object.reverse(Device.browser.BROWSER); //i.e. cr>CHROME,ff>FIREFOX
    // },
    //User friendly descriptors
    getDescriptiveRuntime() {
        return isHybrid() ? "mobile" : "browser";
    },
    /**
     * Gets the device model.
     * <br>Requires cordova device plugin (org.apache.cordova.device)
     * @returns {Boolean}
     */
    getModel() {
        return (window.device ? window.device.model : undefined);
    },
    /**
     * Gets the device's major model version (i.e. '4' for iPhone 4 or 4S)
     * <br>Only tested on iPhone. Some model names use codenames instead of numbers.
     * <br>If the model number is not the first part of the string, the result is NaN
     *
     * @returns {integer|NaN}
     **/
    getMajorModelNumber() {
        if (!window.device)
            return undefined;
        let deviceModel = window.device.model;
        if (sap.ui.Device.os.ios) {
            if (sap.ui.Device.system.phone) {
                deviceModel = deviceModel.replace("iPhone", "");
            }
            else {
                deviceModel = deviceModel.replace("iPad", "");
            }
        }
        return parseInt(deviceModel);
    },
    /**
     * Returns the string version (with '.'s) of the OS version
     *
     * @returns {string}
     **/
    getOSVersion() {
        if (window.device) {
            //from cordova plugin
            return window.device.version;
        }
        else {
            return SAPDevice.os.versionStr;
        }
    },
    getOSMajorVersion() {
        const versionStr = Device.getOSVersion();
        return parseInt(versionStr);
    },
    getOSMinorVersion() {
        const versionStr = Device.getOSVersion();
        const versionArr = versionStr.split(".");
        return parseInt(versionArr[1]) || 0;
    },
    getOSRevisionVersion() {
        const versionStr = Device.getOSVersion();
        const versionArr = versionStr.split(".");
        return parseInt(versionArr[2]) || 0;
    },
    /**
     * Same effect as adding a callback to the 'deviceready' callback to postpone
     * execution until Cordova initialized.
     * On localhost, the callback will be run immidiately
     * @param  {Function} [fn] function to be invoked once the device is ready
     */
    afterDeviceReady(fn) {
        if (fn) {
            deviceReadyPromise.then(fn); //eslint-disable-line promise/catch-or-return
        }
        else {
            console.warn("Please use Device.whenReady() for the Promise API.");
        }
        return deviceReadyPromise;
    },
    isReady() {
        return isReady;
    },
    whenStateChange(cb) {
        if (typeof cb === "function") {
            bchDbgAlert("DEPRECATION: Do not call whenStateChange with a function. Use Connectivity.onSingleStateChange(cb);");
            return onSingleStateChange(cb);
        }
        return whenStateChange();
    },
    onStateChange,
    onSingleStateChange,
    isOnline,
    isOffline,
    whenOnline,
    whenOffline,
    whenReady: whenDeviceReady,
    // Old aliases
    onOnline: whenOnline,
    onOffline: whenOffline,
};
/**
 * Called to set up the device model.
 */
function initDeviceModel() {
    // set device model
    const isOnline = Connectivity.isOnline();
    const deviceExt = {
        isDebug: !!window.dbg,
        isTouch: SAPDevice.support.touch,
        isNoTouch: !SAPDevice.support.touch,
        isPhone: SAPDevice.system.phone,
        isNoPhone: !SAPDevice.system.phone,
        listMode: SAPDevice.system.phone ? "None" : "SingleSelectMaster",
        listItemType: SAPDevice.system.phone ? "Active" : "Inactive",
        envType: getEnvType(),
        isHybrid: isHybrid(),
        isOnline: isOnline,
        isOffline: !isOnline,
        ...SAPDevice
    };
    const deviceModel = new JSONModel(deviceExt);
    Connectivity.onStateChange((isOnline) => {
        deviceExt.isOnline = isOnline;
        deviceExt.isOffline = !isOnline;
        deviceModel.refresh();
    });
    deviceModel.setDefaultBindingMode(BindingMode.OneWay);
    return deviceModel;
}
function initDeviceReady() {
    if (!isHybrid()) {
        isReady = true;
        deviceReadyPromise = Promise.resolve();
    }
    else {
        deviceReadyPromise = new Promise(resolve => {
            document.addEventListener("deviceready", () => {
                console.log("Device is ready");
                isReady = true;
                resolve();
            });
        });
    }
}
export default Device;
bchDbgAdd({ Device });
