import Debug from "bchui5/util/Debug";
import JSONModel from "sap/ui/model/json/JSONModel";
import SAPDevice from "sap/ui/Device";
import * as Connectivity from "bchui5/util/Connectivity";
// import * as ConnectionCheck from "bchui5/util/http/ConnectionCheck";
// , {
// // OnStateChangeHandler, // type
// isOnline,
// isOffline,
// whenOnline,
// whenOffline,
// whenStateChange: whenConnectivity,
// onStateChange,
// onSingleStateChange,
// ConnectivityState
// }
let deviceReadyPromise;
let _isReady = false;
const BindingMode = sap.ui.model.BindingMode;
const system = SAPDevice.system;
const browser = SAPDevice.browser;
const os = SAPDevice.os;
const support = SAPDevice.support;
const media = SAPDevice.media;
export { system, os, browser, support, media };
export let isOnline = Connectivity.isOnline;
export let isOffline = Connectivity.isOffline;
export let whenOnline = Connectivity.whenOnline;
export let whenOffline = Connectivity.whenOffline;
export let onStateChange = Connectivity.onStateChange;
export let onSingleStateChange = Connectivity.onSingleStateChange;
// Keep the BCHUI5 dependencies here minimal.
export function init() {
    const model = initDeviceModel();
    initDeviceReady();
    return model;
}
export function isReady() {
    return _isReady;
}
/**
 * Device or app util methods.
 * All functions should be completely stateless.
 * Warning: Model number functions depend on the cordova device plugin (org.apache.cordova.device)
 */
export function whenDeviceReady() {
    return deviceReadyPromise;
}
export const whenReady = whenDeviceReady;
/**
 * Checks whether an app is running on localhost
 * @returns {Boolean}
 */
export function isLocalhost() {
    return window.location.hostname === "localhost";
}
/**
 * Checks whether an app page is served from file.
 * Different than isHybrid because a launchpad app may have window.cordova
 * but not served from file.
 * This one is also safe to call before cordova is ready, where window.cordova may not be.
 * @returns {Boolean}
 */
export function isFileApp() {
    return window.location.protocol === "file:";
}
export function isFileHybrid() {
    return window.location.protocol === "file:";
}
// export function isEmulator() {
//   const { platform } = window.device
//   if (platform === "iOS") {
//     return jQuery.sap.device.model === "x86_64";
//   }
//   else {
//     alert("Cordova Util isEmulator() not yet supported on platform: " + platform);
//     return false;
//   }
// }
/**
 * Checks whether an app is hybrid, if it meets either condition
 * of having window.cordova or being served from file.
 * Returns true for an https served html like launchpad inside a cordova container.
 * Be careful with the expected result of launchpad based apps (true).
 * @returns {Boolean}
 */
export function isHybrid() {
    return window.location.protocol === "file:" || !!window.cordova;
}
/**
 * A hybrid launchpad app (i.e. inside Fiori launchpad)
 * will be served over http/https, rather than file.
 * @returns {Boolean}
 */
export function isHybridLaunchpadApp() {
    if (isFileApp()) {
        return false;
    }
    // else http/https
    try {
        if (window.cordova || window.parent.cordova)
            return true;
    }
    catch (error) {
        // catch and ignore SecurityError when running in an iframe on portal
    }
    // If (window.parent.cordova) return true; //works on iOS 7 & 8
    // since we can't check for window.cordova or window.parent.cordova on iOS9
    // our solution gets a bit hacky.
    if (system.desktop)
        return false;
    // If we're here then we're on a phone or tablet.
    // But it could be mobile browser rather than launchpad.
    // It's a bit hacky but check for SMP urls.
    if (window.location.hostname.match(/mdm.bchydro.com/))
        return true;
    if (window.location.hostname.match(/smp.bchydro.bc.ca/))
        return true;
}
/**
 * Checks whether an app is hybrid or running on localhost
 * @returns {Boolean}
 */
export function isHybridOrLocalhost() {
    return isHybrid() || isLocalhost();
}
/**
 * Checks whether an app is file: based or running on localhost
 * Different from isHybridOrLocalhost as it will be false
 * for remote served launchpad apps
 * @returns {Boolean}
 */
export function isFileOrLocalhost() {
    return isFileApp() || isLocalhost();
}
/**
 * Checks whether an app is desktop browser based.
 * false for localhost and hosted Launchpad apps.
 * @returns {Boolean}
 */
export function isDesktopBrowser() {
    return !isHybridOrLocalhost();
}
export function isDesktop() {
    return !isHybridOrLocalhost();
}
export function getRuntime() {
    // These runtimes match those used in app.json
    // Which is why some may not be semantically correct (i.e. hybrid vs hybrid launchpad)
    if (isFileApp()) {
        return "hybrid";
    }
    else if (isLocalhost()) {
        return "localhost";
    }
    else {
        // Also applies to Fiori launchpad on browser and on SAP Apps
        return "server";
    }
}
export function getEnvType() {
    return getRuntime();
}
/**
 * Same effect as adding a callback to the 'deviceready' callback to postpone
 * execution until Cordova initialized.
 * On localhost, the callback will be run immediately
 * @param  {Function} [fn] function to be invoked once the device is ready
 */
export function afterDeviceReady(fn) {
    if (fn) {
        deviceReadyPromise.then(fn);
    }
    else {
        console.warn("Please use Device.whenReady() for the Promise API.");
    }
    return deviceReadyPromise;
}
export function getDescriptiveName() {
    if (isHybrid()) {
        return os.name + " " + os.versionStr;
    }
    else {
        return browser.name.toUpperCase() + " " + browser.versionStr;
    }
}
export function getDescriptiveRuntime() {
    return isHybrid() ? "mobile" : "browser";
}
/**
 * Gets the device model.
 * <br>Requires cordova device plugin (org.apache.cordova.device)
 * @returns {Boolean}
 */
export function getModel() {
    return (window.device ? window.device.model : undefined);
}
/**
 * Gets the device's major model version (i.e. '4' for iPhone 4 or 4S)
 * <br>Only tested on iPhone. Some model names use code-names instead of numbers.
 * <br>If the model number is not the first part of the string, the result is NaN
 *
 * @returns {integer|NaN}
 */
export function getMajorModelNumber() {
    if (!window.device)
        return undefined;
    let deviceModel = window.device.model;
    if (os.ios) {
        if (system.phone) {
            deviceModel = deviceModel.replace("iPhone", "");
        }
        else {
            deviceModel = deviceModel.replace("iPad", "");
        }
    }
    return parseInt(deviceModel, 10);
}
/**
 * Returns the string version (with '.'s) of the OS version
 *
 * @returns {string}
 */
export function getOSVersion() {
    if (window.device) {
        return window.device.version; // From cordova plugin
    }
    else {
        return os.versionStr;
    }
}
export function getOSMajorVersion() {
    const versionStr = Device.getOSVersion();
    return parseInt(versionStr, 10);
}
export function getOSMinorVersion() {
    const versionStr = Device.getOSVersion();
    const versionArr = versionStr.split(".");
    return parseInt(versionArr[1], 10) || 0;
}
export function getOSRevisionVersion() {
    const versionStr = Device.getOSVersion();
    const versionArr = versionStr.split(".");
    return parseInt(versionArr[2], 10) || 0;
}
export function whenStateChange() {
    const callback = arguments[0];
    if (typeof callback === "function") {
        Debug.alert("DEPRECATION: Do not call whenStateChange with a function. Use Connectivity.onSingleStateChange(cb);");
        return whenStateChange()
            .then(async (state) => {
            await callback(state);
            return state;
        });
    }
    else {
        return whenStateChange();
    }
}
// Old aliases
export let onOnline = whenOnline;
export let onOffline = whenOffline;
const Device = {
    ...SAPDevice,
    // system,
    // os,
    // browser,
    // support,
    // media,
    init,
    isDesktop,
    isDesktopBrowser,
    // isEmulator,
    isFileOrLocalhost,
    isFileApp,
    isFileHybrid,
    isHybrid,
    isHybridLaunchpadApp,
    isHybridOrLocalhost,
    isLocalhost,
    getRuntime,
    getEnvType,
    getDescriptiveName,
    getDescriptiveRuntime,
    getModel,
    getMajorModelNumber,
    getOSVersion,
    getOSMajorVersion,
    getOSMinorVersion,
    getOSRevisionVersion,
    afterDeviceReady,
    isReady,
    whenStateChange,
    onStateChange,
    onSingleStateChange,
    isOnline,
    isOffline,
    whenOnline,
    whenOffline,
    whenDeviceReady,
    whenReady,
    // Old aliases
    onOnline: whenOnline,
    onOffline: whenOffline,
};
/**
 * Called to set up the device model.
 */
function initDeviceModel() {
    const { phone } = system;
    const { touch } = support;
    // set device model
    const isOnline = Connectivity.isOnline(); // tslint:disable-line no-shadowed-variable
    const deviceExt = {
        isDebug: Debug.isEnabled(),
        isTouch: touch,
        isNoTouch: !touch,
        isPhone: phone,
        isNoPhone: !phone,
        listMode: phone ? "None" : "SingleSelectMaster",
        listItemType: phone ? "Active" : "Inactive",
        envType: getEnvType(),
        isHybrid: isHybrid(),
        isOnline,
        isOffline: !isOnline,
        ...SAPDevice
    };
    const deviceModel = new JSONModel(deviceExt);
    Connectivity.onStateChange((isOnline) => {
        deviceExt.isOnline = isOnline;
        deviceExt.isOffline = !isOnline;
        deviceModel.refresh();
    });
    deviceModel.setDefaultBindingMode(BindingMode.OneWay);
    return deviceModel;
}
function initDeviceReady() {
    if (!isHybrid()) {
        _isReady = true;
        deviceReadyPromise = Promise.resolve();
    }
    else {
        deviceReadyPromise = new Promise(resolve => {
            document.addEventListener("deviceready", () => {
                console.log("Device is ready");
                _isReady = true;
                resolve();
            });
        });
    }
}
export default Device;
Debug.add({ Device });
